@@ -314,6 +314,16 @@ def all_errors(request: pytest.FixtureRequest, monkeypatch: pytest.MonkeyPatch)
             'Field "project.readme.content-type" missing',
             id="Missing content-type for readme",
         ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                readme = { file = "pyproject.toml" }
+            """,
+            'Field "project.readme.content-type" missing',
+            id="Missing content-type for readme file",
+        ),
         pytest.param(
             """
                 [project]
@@ -780,6 +790,78 @@ def all_errors(request: pytest.FixtureRequest, monkeypatch: pytest.MonkeyPatch)
             "Setting \"project.license\" to an SPDX license expression is not compatible with 'License ::' classifiers",
             id="SPDX license and License trove classifiers",
         ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-names = ["is"]
+            """,
+            "\"import-names\" contains a Python keyword, which is not a valid import name, got 'is'",
+            id="Setting import-names to keyword",
+        ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-namespaces = ["from"]
+            """,
+            "\"import-namespaces\" contains a Python keyword, which is not a valid import name, got 'from'",
+            id="Setting import-namespaces to keyword",
+        ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-names = ["2two"]
+            """,
+            "\"import-names\" contains '2two', which is not a valid identifier",
+            id="Setting import-names invalid identifier",
+        ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-namespaces = ["3"]
+            """,
+            "\"import-namespaces\" contains '3', which is not a valid identifier",
+            id="Setting import-namespaces to invalid identifier",
+        ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-names = ["one", "two"]
+                import-namespaces = ["one", "three"]
+            """,
+            "\"project.import-names\" overlaps with 'project.import-namespaces': {'one'}",
+            id="Matching entry in import-names and import-namespaces",
+        ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-names = ["one; private", "two"]
+                import-namespaces = ["one", "three    ;   private"]
+            """,
+            "\"project.import-names\" overlaps with 'project.import-namespaces': {'one'}",
+            id="Matching entry in import-names and import-namespaces with private tags",
+        ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-names = ["one.two"]
+            """,
+            "\"project.import-namespaces\" is missing 'one', but submodules are present elsewhere",
+            id="Matching entry in import-names and import-namespaces",
+        ),
     ],
 )
 def test_load(
@@ -886,6 +968,22 @@ def test_load(
             ],
             id="Four errors including extra keys",
         ),
+        pytest.param(
+            """
+                [project]
+                name = 'test'
+                version = "0.1.0"
+                import-names = ["test", "other"]
+                import-namespaces = ["other.one.two", "invalid name", "not; public"]
+            """,
+            [
+                "\"import-namespaces\" contains 'invalid name', which is not a valid identifier",
+                "\"import-namespaces\" contains an ending tag other than '; private', got 'not; public'",
+                "\"import-namespaces\" contains a Python keyword, which is not a valid import name, got 'not; public'",
+                "\"project.import-namespaces\" is missing 'other.one', but submodules are present elsewhere",
+            ],
+            id="Multiple errors related to names/namespaces",
+        ),
     ],
 )
 def test_load_multierror(
@@ -928,7 +1026,7 @@ def test_load_multierror(
                 version = "0.1.0"
                 license = 'MIT'
             """,
-            'Setting "project.license" to an SPDX license expression is supported only when emitting metadata version >= 2.4',
+            'Setting "project.license" to an SPDX license expression is only supported when emitting metadata version >= 2.4',
             "2.3",
             id="SPDX with metadata_version 2.3",
         ),
@@ -939,10 +1037,32 @@ def test_load_multierror(
                 version = "0.1.0"
                 license-files = ['README.md']
             """,
-            '"project.license-files" is supported only when emitting metadata version >= 2.4',
+            '"project.license-files" is only supported when emitting metadata version >= 2.4',
             "2.3",
             id="license-files with metadata_version 2.3",
         ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-names = ['one']
+            """,
+            '"project.import-names" is only supported when emitting metadata version >= 2.5',
+            "2.4",
+            id="import-names with metadata_version 2.4",
+        ),
+        pytest.param(
+            """
+                [project]
+                name = "test"
+                version = "0.1.0"
+                import-namespaces = ['one']
+            """,
+            '"project.import-namespaces" is only supported when emitting metadata version >= 2.5',
+            "2.4",
+            id="import-names with metadata_version 2.4",
+        ),
     ],
 )
 def test_load_with_metadata_version(
@@ -1021,6 +1141,7 @@ def test_value(after_rfc: bool, monkeypatch: pytest.MonkeyPatch) -> None:
         ("Example!", None),
     ]
     assert metadata.maintainers == [
+        ("Emailless", None),
         ("Other Example", "other@example.com"),
     ]
     assert metadata.keywords == ["trampolim", "is", "interesting"]
@@ -1060,6 +1181,25 @@ def test_value(after_rfc: bool, monkeypatch: pytest.MonkeyPatch) -> None:
     ]
 
 
+@pytest.mark.parametrize("after_rfc", [False, True])
+def test_value_25(after_rfc: bool, monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.chdir(DIR / "packages/metadata-2.5")
+    with open("pyproject.toml", "rb") as f:
+        metadata = pyproject_metadata.StandardMetadata.from_pyproject(tomllib.load(f))
+
+    if after_rfc:
+        metadata.as_rfc822()
+
+    assert metadata.auto_metadata_version == "2.5"
+
+    assert isinstance(metadata.license, str)
+    assert metadata.license == "MIT"
+    assert metadata.license_files == [pathlib.Path("LICENSE")]
+
+    assert metadata.import_names == ["metadata25"]
+    assert metadata.import_namespaces is None
+
+
 def test_read_license(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.chdir(DIR / "packages/full-metadata2")
     with open("pyproject.toml", "rb") as f:
@@ -1117,6 +1257,7 @@ def test_as_json(monkeypatch: pytest.MonkeyPatch) -> None:
         "description_content_type": "text/markdown",
         "keywords": ["trampolim", "is", "interesting"],
         "license": "some license text",
+        "maintainer": "Emailless",
         "maintainer_email": "Other Example <other@example.com>",
         "metadata_version": "2.1",
         "name": "full_metadata",
@@ -1143,6 +1284,20 @@ def test_as_json(monkeypatch: pytest.MonkeyPatch) -> None:
     }
 
 
+def test_readme_text() -> None:
+    pyproject = pyproject_metadata.StandardMetadata.from_pyproject(
+        {
+            "project": {
+                "name": "foo",
+                "version": "1.2.3",
+                "readme": {"text": "onetwothree", "content-type": "text/plain"},
+            }
+        }
+    )
+    assert pyproject.readme
+    assert pyproject.readme.text == "onetwothree"
+
+
 def test_as_rfc822(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.chdir(DIR / "packages/full-metadata")
 
@@ -1157,6 +1312,7 @@ def test_as_rfc822(monkeypatch: pytest.MonkeyPatch) -> None:
         ("Keywords", "trampolim,is,interesting"),
         ("Author", "Example!"),
         ("Author-Email", "Unknown <example@example.com>"),
+        ("Maintainer", "Emailless"),
         ("Maintainer-Email", "Other Example <other@example.com>"),
         ("License", "some license text"),
         ("Classifier", "Development Status :: 4 - Beta"),
@@ -1183,11 +1339,53 @@ def test_as_rfc822(monkeypatch: pytest.MonkeyPatch) -> None:
     assert core_metadata.get_payload() == "some readme 👋\n"
 
 
+def test_rfc822_empty_import_name() -> None:
+    metadata = pyproject_metadata.StandardMetadata.from_pyproject(
+        {"project": {"name": "test", "version": "0.1.0", "import-names": []}}
+    )
+    assert metadata.import_names == []
+    assert metadata.import_namespaces is None
+
+    core_metadata = metadata.as_rfc822()
+    assert core_metadata.items() == [
+        ("Metadata-Version", "2.5"),
+        ("Name", "test"),
+        ("Version", "0.1.0"),
+        ("Import-Name", ""),
+    ]
+
+
+def test_rfc822_full_import_name() -> None:
+    metadata = pyproject_metadata.StandardMetadata.from_pyproject(
+        {
+            "project": {
+                "name": "test",
+                "version": "0.1.0",
+                "import-names": ["one", "two"],
+                "import-namespaces": ["three"],
+            }
+        }
+    )
+    assert metadata.import_names == ["one", "two"]
+    assert metadata.import_namespaces == ["three"]
+
+    core_metadata = metadata.as_rfc822()
+    assert core_metadata.items() == [
+        ("Metadata-Version", "2.5"),
+        ("Name", "test"),
+        ("Version", "0.1.0"),
+        ("Import-Name", "one"),
+        ("Import-Name", "two"),
+        ("Import-Namespace", "three"),
+    ]
+
+
 def test_as_json_spdx(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.chdir(DIR / "packages/spdx")
 
     with open("pyproject.toml", "rb") as f:
         metadata = pyproject_metadata.StandardMetadata.from_pyproject(tomllib.load(f))
+
     core_metadata = metadata.as_json()
     assert core_metadata == {
         "license_expression": "MIT OR GPL-2.0-or-later OR (FSFUL AND BSD-2-Clause)",
@@ -1259,9 +1457,12 @@ def test_license_file_24(
     pre_spdx = (
         metadata_version in pyproject_metadata.constants.PRE_SPDX_METADATA_VERSIONS
     )
-    with contextlib.nullcontext() if pre_spdx else pytest.warns(  # type: ignore[attr-defined]
-        pyproject_metadata.errors.ConfigurationWarning
-    ):
+    ctx = (
+        contextlib.nullcontext()
+        if pre_spdx
+        else pytest.warns(pyproject_metadata.errors.ConfigurationWarning)
+    )
+    with ctx:
         metadata = pyproject_metadata.StandardMetadata.from_pyproject(
             {
                 "project": {
@@ -1278,6 +1479,30 @@ def test_license_file_24(
     else:
         assert "License-File: LICENSE.txt" in message
 
+    bmessage = bytes(metadata.as_rfc822())
+    if metadata_version in pyproject_metadata.constants.PRE_SPDX_METADATA_VERSIONS:
+        assert b"License-File: LICENSE.txt" not in bmessage
+    else:
+        assert b"License-File: LICENSE.txt" in bmessage
+
+
+def test_license_file_broken(monkeypatch: pytest.MonkeyPatch) -> None:
+    monkeypatch.chdir(DIR / "packages/broken_license")
+
+    metadata = pyproject_metadata.StandardMetadata.from_pyproject(
+        {
+            "project": {
+                "name": "broken_license",
+                "version": "0.1.0",
+                "license": {"file": "LICENSE"},
+            },
+        },
+    )
+    message = str(metadata.as_rfc822())
+    assert "License-File: LICENSE" not in message
+    bmessage = bytes(metadata.as_rfc822())
+    assert b"License-File: LICENSE" not in bmessage
+
 
 def test_as_rfc822_dynamic(monkeypatch: pytest.MonkeyPatch) -> None:
     monkeypatch.chdir(DIR / "packages/dynamic-description")
@@ -1385,19 +1610,19 @@ def test_as_rfc822_invalid_dynamic() -> None:
     )
     with pytest.raises(
         pyproject_metadata.ConfigurationError,
-        match="Field cannot be set as dynamic metadata: name",
+        match="Metadata field 'name' cannot be declared dynamic",
     ):
         metadata.as_rfc822()
-    metadata.dynamic_metadata = ["version"]
+    metadata.dynamic_metadata = ["Version"]
     with pytest.raises(
         pyproject_metadata.ConfigurationError,
-        match="Field cannot be set as dynamic metadata: version",
+        match="Metadata field 'Version' cannot be declared dynamic",
     ):
         metadata.as_rfc822()
     metadata.dynamic_metadata = ["unknown"]
     with pytest.raises(
         pyproject_metadata.ConfigurationError,
-        match="Field is not known: unknown",
+        match="Unknown metadata field 'unknown' cannot be declared dynamic",
     ):
         metadata.as_rfc822()
 
@@ -1417,15 +1642,18 @@ def test_as_rfc822_mapped_dynamic() -> None:
 def test_as_rfc822_missing_version() -> None:
     metadata = pyproject_metadata.StandardMetadata(name="something")
     with pytest.raises(
-        pyproject_metadata.ConfigurationError, match="Missing version field"
+        pyproject_metadata.ConfigurationError,
+        match=re.escape('Field "project.version" missing'),
     ):
         metadata.as_rfc822()
 
 
-def test_stically_defined_dynamic_field() -> None:
+def test_statically_defined_dynamic_field() -> None:
     with pytest.raises(
         pyproject_metadata.ConfigurationError,
-        match='Field "project.version" declared as dynamic in "project.dynamic" but is defined',
+        match=re.escape(
+            'Field "project.version" declared as dynamic in "project.dynamic" but is defined'
+        ),
     ):
         pyproject_metadata.StandardMetadata.from_pyproject(
             {
@@ -1444,8 +1672,8 @@ def test_stically_defined_dynamic_field() -> None:
     "value",
     [
         "<3.10",
-        ">3.7,<3.11",
-        ">3.7,<3.11,!=3.8.4",
+        ">3.8,<3.11",
+        ">3.8,<3.11,!=3.8.4",
         "~=3.10,!=3.10.3",
     ],
 )
